@@ -5,6 +5,7 @@ import re
 import os
 import xml.etree.ElementTree as ET
 import copy
+import json
 import hashlib
 
 from svtplay_dl.log import log
@@ -37,7 +38,7 @@ class Svtplay(Service, OpenGraphThumbMixin):
         if re.match("^[0-9]+$", vid):
             old = True
 
-        url = "http://www.svt.se/videoplayer-api/video/%s" % vid
+        url = "http://api.svt.se/videoplayer-api/video/%s" % vid
         data = self.http.request("get", url)
         if data.status_code == 404:
             yield ServiceError("Can't get the json file for %s" % url)
@@ -81,11 +82,20 @@ class Svtplay(Service, OpenGraphThumbMixin):
             return
 
         for i in data["videoReferences"]:
+            parse = urlparse(i["url"])
+            query = parse_qs(parse.query)
             if i["format"] == "hls" or i["format"] == "ios":
                 streams = hlsparse(self.options, self.http.request("get", i["url"]), i["url"])
                 if streams:
                     for n in list(streams.keys()):
                         yield streams[n]
+                if "alt" in query and len(query["alt"]) > 0:
+                    alt = self.http.get(query["alt"][0])
+                    if alt:
+                        streams = hlsparse(self.options, self.http.request("get", alt.request.url), alt.request.url)
+                        if streams:
+                            for n in list(streams.keys()):
+                                yield streams[n]
             if i["format"] == "hds" or i["format"] == "flash":
                 match = re.search(r"\/se\/secure\/", i["url"])
                 if not match:
@@ -93,12 +103,40 @@ class Svtplay(Service, OpenGraphThumbMixin):
                     if streams:
                         for n in list(streams.keys()):
                             yield streams[n]
+                    if "alt" in query and len(query["alt"]) > 0:
+                        alt = self.http.get(query["alt"][0])
+                        if alt:
+                            streams = hdsparse(self.options, self.http.request("get", alt.request.url, params={"hdcore": "3.7.0"}), alt.request.url)
+                            if streams:
+                                for n in list(streams.keys()):
+                                    yield streams[n]
             if i["format"] == "dash264":
                 streams = dashparse(self.options, self.http.request("get", i["url"]), i["url"])
                 if streams:
                     for n in list(streams.keys()):
                         yield streams[n]
 
+                if "alt" in query and len(query["alt"]) > 0:
+                    alt = self.http.get(query["alt"][0])
+                    streams = dashparse(self.options, self.http.request("get", alt.request.url), alt.request.url)
+                    if streams:
+                        for n in list(streams.keys()):
+                            yield streams[n]
+
+            if i["format"] == "dashhbbtv":
+                streams = dashparse(self.options, self.http.request("get", i["url"]), i["url"])
+                if streams:
+                    for n in list(streams.keys()):
+                        yield streams[n]
+
+                if "alt" in query and len(query["alt"]) > 0:
+                    alt = self.http.get(query["alt"][0])
+                    if alt:
+                        streams = dashparse(self.options, self.http.request("get", alt.request.url), alt.request.url)
+                        if streams:
+                            for n in list(streams.keys()):
+                                yield streams[n]
+
 
     def find_video_id(self):
         match = re.search('data-video-id="([^"]+)"', self.get_urldata())
@@ -149,17 +187,68 @@ class Svtplay(Service, OpenGraphThumbMixin):
 
         return None
 
+    def _last_chance(self, videos, page, maxpage=2):
+        if page > maxpage:
+            return videos
+
+        res = self.http.get("http://www.svtplay.se/sista-chansen?sida=%s" % page)
+        match = re.search("__reduxStore'] = ({.*});", res.text)
+        if not match:
+            return videos
+
+        dataj = json.loads(match.group(1))
+        pages = dataj["gridPage"]["pagination"]["totalPages"]
+
+        for i  in dataj["gridPage"]["content"]:
+            videos.append(i["contentUrl"])
+        page += 1
+        self._last_chance(videos, page, pages)
+        return videos
+
+    def _genre(self, jansson):
+        videos = []
+        for i in jansson["clusterPage"]["content"]["clips"]:
+            videos.append(i["contentUrl"])
+        return videos
+
     def find_all_episodes(self, options):
-        match = re.search(r'<link rel="alternate" type="application/rss\+xml" [^>]*href="([^"]+)"',
+        parse = urlparse(self._url)
+        
+        if len(parse.path) > 7 and parse.path[-7:] == "rss.xml":
+            match = self.url
+        else:
+            match = re.search(r'<link rel="alternate" type="application/rss\+xml" [^>]*href="([^"]+)"',
                           self.get_urldata())
+            if match:
+                match = match.group(1)
+            
         if match is None:
-            match = re.findall(r'a class="play[^"]+"\s+href="(/video[^"]+)"', self.get_urldata())
-            if not match:
+            videos = []
+            match = re.search("__reduxStore'] = ({.*});", self.get_urldata())
+            if match:
+                dataj = json.loads(match.group(1))
+            else:
                 log.error("Couldn't retrieve episode list")
                 return
-            episodes = [urljoin("http://www.svtplay.se", x) for x in match]
+            if re.search("sista-chansen", parse.path):
+                videos = self._last_chance(videos, 1)
+            elif re.search("/genre", parse.path):
+                videos = self._genre(dataj)
+            else:
+                items = dataj["videoTitlePage"]["realatedVideoTabs"]
+                for i in items:
+                    if "sasong" in i["slug"]:
+                        for n in i["videos"]:
+                            if n["url"] not in videos:
+                                videos.append(n["url"])
+                    if "senast" in i["slug"]:
+                        for n in i["videos"]:
+                            if n["url"] not in videos:
+                                videos.append(n["url"])
+
+            episodes = [urljoin("http://www.svtplay.se", x) for x in videos]
         else:
-            data = self.http.request("get", match.group(1)).content
+            data = self.http.request("get", match).content
             xml = ET.XML(data)
 
             episodes = [x.text for x in xml.findall(".//item/link")]
